#!/usr/bin/env python

#
#Copyrigt (c) 2008
#		Matt Smith <melkor@orangepalantir.org>
#
#This file is part of GraphCutie,This file is part of Foobar.
#
#GraphCutie is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#GraphCutie is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with GraphCutie, in the file COPYING.  
#If not, see <http://www.gnu.org/licenses/>.
#
#GraphCutie uses both python and PyQT, GPL version.  The copyright notices
#can be found in copyright_notice.txt


"""Central Widget, it is a stacked widget with a tab bar."""
try:
	from PyQt4 import QtGui,QtCore,QtSvg
except:
	print "you need PyQt4 to use this application as a python script"
import sys,os
from Graph import SVGWidget
from CalcGui import GuiCalc
from Configs import WinConfigWidget,PlotConfigWidget
from FunctionDisplay import FunctionDisplay
from QtDisplay import PATHNAME

class CentralWindow(QtGui.QWidget):
	"""This is really the main window it is divided into three parts it has a tab bar, a stack widget and a function input panel.
	the stack widget is broken down into the various classes here."""
	def __init__(self,calc,svgplot,parent=None):
		QtGui.QWidget.__init__(self,parent)
		self.Calculator = calc
		layout = QtGui.QVBoxLayout()
		
		self.Tabs = TabWindow(self)
		layout.addWidget(self.Tabs)
		
		self.Stacks = QtGui.QStackedWidget(self)
		self.Stacks.connect(self.Tabs,QtCore.SIGNAL("currentChanged( int )"),QtCore.SLOT("setCurrentIndex(int)"))
		self.Stacks.setFrameStyle(QtGui.QFrame.Box)
		self.Stacks.setLineWidth(1)
		self.Stacks.setMidLineWidth(0)
		self.Stacks.setFrameShadow(QtGui.QFrame.Plain)
		
		self.MainPlot = SVGWidget(svgplot,self)
		self.Tabs.connect(self.MainPlot,QtCore.SIGNAL("attention(int)"),QtCore.SLOT("setCurrentIndex(int)"))
		self.MainPlot.connect(parent.menuWidget(),QtCore.SIGNAL("imageSaveRequest( QString )"),self.MainPlot.SaveImage)
		self.GC = GuiCalc(self)
		self.GC.connect(parent.menuWidget(),QtCore.SIGNAL("Calculate( QString )"),self.GC.ExecScriptCmd)
		self.PC = PlotConfigWidget(self)
		self.SF = FunctionDisplay(self)
		layout.addWidget(self.Stacks)
		
		self.Stacks.addWidget(self.GC)
		self.Stacks.addWidget(self.MainPlot)
		self.Stacks.addWidget(WinConfigWidget(self))
		self.Stacks.addWidget(self.PC)
		self.Stacks.addWidget(self.SF)
		
		self.Functions = []
		
		self.Functions.append(PlotInputWidget('F1','red'))
		self.Functions.append(PlotInputWidget('F2','blue'))
		self.Functions.append(PlotInputWidget('F3','green'))
		self.Functions.append(PlotInputWidget('F4','black'))
		
		self.connect(parent.menuWidget(),QtCore.SIGNAL("LoadPlot( QString )"),self.ScriptPlot)
		self.connect(parent.menuWidget(),QtCore.SIGNAL("PlotRequest()"),self.MainPlot.Graph.Plot)
		
		for item in self.Functions:
			self.connect(item,QtCore.SIGNAL("plotrequest ()"),self.MainPlot.Graph.Plot)
		layout.addWidget(FunctionCombo(self.Functions,self))
		self.setLayout(layout)
	def ErrorMessage(self,qst):
		"""When any widget catches an exception the message will be displayed on the calculator"""
		self.GC.ErrorMessage(qst)
		self.Tabs.setCurrentIndex(0)
	def PlotFunction(self,function):
		"""This function will put a random function into the box and plot it."""
		self.Functions[0].FunctionBox.setText(function)
		self.MainPlot.Graph.Plot()
	def ScriptPlot(self,command):
		index = int(command[1])-1
		self.Functions[index].FunctionBox.setText(command[6:])
		
class PlotInputWidget(QtGui.QWidget):
	def __init__(self,fname,color,parent=None):
		QtGui.QWidget.__init__(self,parent)
		self.FunctionBox = QtGui.QLineEdit(self)
		self.FunctionBox.setFixedHeight(25)
		self.Function = QtGui.QPushButton("PLOT",self)
		self.Function.setFixedHeight(20)
		self.fname = "%s(x)="%fname
		self.ColorSelect = ColorButton(color,self)
		sublayout = QtGui.QHBoxLayout()
		sublayout.setContentsMargins(0,0,0,0)
		self.FLabel = QtGui.QLabel(self.fname,self)
		self.FLabel.setFixedHeight(20)
		sublayout.addWidget(self.FLabel)
		sublayout.addWidget(self.FunctionBox)
		sublayout.addWidget(self.Function)
		sublayout.addWidget(self.ColorSelect)
		self.connect(self.Function,QtCore.SIGNAL("clicked()"),self.ReqPlot)
		self.connect(self.FunctionBox,QtCore.SIGNAL("returnPressed()"),self.ReqPlot)
		self.setLayout(sublayout)
	def PlotReply(self):
		if len(str(self.FunctionBox.text()))>0:
			ret_fun = "%s%s"%(self.fname,self.FunctionBox.text())
		else:
			ret_fun = None
		return ret_fun,self.ColorSelect.GetColor()
	def ReqPlot(self):
		self.emit(QtCore.SIGNAL("plotrequest ()"))


class TabWindow(QtGui.QTabBar):
	def __init__(self,parent=None):
		QtGui.QTabBar.__init__(self,parent)
		self.addTab("Calculator")
		self.addTab("Graph")
		self.addTab("settings")
		self.addTab("Plot Range")
		self.addTab("Function List")


class ColorButton(QtGui.QLabel):
	def __init__(self,color,parent):
		QtGui.QLabel.__init__(self,parent)
		self.setFixedHeight(20)
		self.color = QtGui.QColor(color)
		self.bgcolor = QtGui.QPixmap(20,20)
		self.bgcolor.fill(self.color)
		self.setPixmap(self.bgcolor)
	def mousePressEvent(self,e):
		self.color = QtGui.QColorDialog.getColor(self.color)
		self.bgcolor.fill(self.color)
		self.setPixmap(self.bgcolor)
	def GetColor(self):
		return "#%2.2x%2.2x%2.2x"%self.color.getRgb()[0:-1]

class FunctionCombo(QtGui.QFrame):
	def __init__(self,flist,parent=None):
		QtGui.QFrame.__init__(self,parent)
		self.setFrameStyle(1)
		self.Index = 0
		self.Stacks = QtGui.QStackedWidget(self)
		for item in flist:
			self.Stacks.addWidget(item)
		self.Up = QtGui.QPushButton(self)
		pm = QtGui.QIcon(os.path.join(PATHNAME,'up.png'))
		self.Up.setIcon(pm)
		self.Up.setFixedHeight(10)
		self.Up.setFocusPolicy(QtCore.Qt.NoFocus)
		dm = QtGui.QIcon(os.path.join(PATHNAME,'down.png'))
		self.Down = QtGui.QPushButton(self)
		self.Down.setIcon(dm)
		self.Down.setFixedHeight(10)
		self.Down.setFocusPolicy(QtCore.Qt.NoFocus)
		
		self.connect(self.Up,QtCore.SIGNAL("clicked ()"),self.Iu)
		self.connect(self.Down,QtCore.SIGNAL("clicked ()"),self.Id)
		layout = QtGui.QHBoxLayout()
		sublayout = QtGui.QVBoxLayout()
		sublayout.addWidget(self.Up)
		sublayout.addWidget(self.Down)
		layout.addLayout(sublayout)
		layout.addWidget(self.Stacks)
		self.setLayout(layout)
		self.setFixedHeight(40)
	def increment(self,d):
		if d=='u':
			self.Index += 1
			if self.Index>=self.Stacks.count():
				self.Index = 0
		if d=='d':
			self.Index -= 1
			if self.Index<0:
				self.Index=self.Stacks.count() -1
		self.Stacks.setCurrentIndex(self.Index)
	def Iu(self):
		self.increment('d')
	def Id(self):
		self.increment('u')